"""
author : kerwin
function : personal settings
note : 用于修改所有文件的默认配置信息

警示： 以后再配置默认路径的时候，不要带上尾巴上的‘/’.  实际应用中看着不舒服。。。
        ("D:/自如/" 由于历史原因，不想改了。。。) //k200207:已经修改回来了
"""


"""
mac 和 linux 的路径除了开头的/Users 替换成 /root 外，其他是一样的。
"""



# 自动判断该系统是 ’windows' 还是 ‘mac’
import platform
import sys, os
    # win
if platform.system() == "Windows":
    PYTHON = "python"
    HOST = "127.0.0.1"
    FILE_PATH_FOR_DESKTOP = "C:/Users/Administrator/Desktop"
    FILE_PATH_FOR_MYCODE = "D:/kerwin/MyCode"
    FILE_PATH_FOR_ZIRU_CODE = "D:/kerwin/MyCode/Business/Ziru"
    FILE_PATH_FOR_ZIRU = "D:/自如"

    # mac
elif platform.system() == "Darwin":
    PYTHON = "python3"
    HOST = "127.0.0.1" # 本地ip  (mac上的代码默认用本地ip，若个别脚本有特别需要，可以设置成REMOTE_HOST)
    # HOST = "121.40.240.153" # 阿里云的ip
    REMOTE_HOST = "121.40.240.153" # 阿里云的ip
    FILE_PATH_FOR_DESKTOP = "/Users/kerwin/Desktop"
    FILE_PATH_FOR_MYCODE = "/Users/kerwin/MyBox/MyCode"
    FILE_PATH_FOR_ZIRU_CODE = "/Users/kerwin/MyBox/MyCode/Business/Ziru"
    FILE_PATH_FOR_ZIRU = "/Users/kerwin/MyBox/工作相关/自如"

    # linux
elif platform.system() == "Linux":
    PYTHON = "python3"
    HOST = "127.0.0.1"
    FILE_PATH_FOR_DESKTOP = "/root/kerwin/Desktop"
    FILE_PATH_FOR_MYCODE = "/root/kerwin/MyBox/MyCode"
    FILE_PATH_FOR_ZIRU_CODE = "/root/kerwin/MyBox/MyCode/Business/Ziru"
    FILE_PATH_FOR_ZIRU = "/root/kerwin/MyBox/工作相关/自如"

# 检查是否存在上述文件路径; (没有则抛出异常)
error_path = []
for path in [FILE_PATH_FOR_DESKTOP, FILE_PATH_FOR_MYCODE, FILE_PATH_FOR_ZIRU_CODE, FILE_PATH_FOR_ZIRU]:
    if not os.path.exists(path):
        error_path.append(path)
if error_path:
    raise Exception(
        "[kk异常]:\n不存在以下文件路径, 建议创建后重试:\n" + "\n".join(error_path)
    )




# 自动搜索kw618库包的位置
site_pkgs_path = [path for path in sys.path if "site-packages" in path]
FILE_PATH_FOR_KW618 = None
for path in site_pkgs_path:
    if os.path.exists(f"{path}/kw618"):
        FILE_PATH_FOR_KW618 = f"{path}/kw618"
        break
FILE_PATH_FOR_KW618 = '/Users/kerwin/MyBox/MyKernel/kw618' # <toggle>
if FILE_PATH_FOR_KW618 is None:
    raise Exception("\n\n[kk异常]:  在本地 site-packages 路径中找不到kw618库, 尝试使用 pip3 install kw618\n\n")




if __name__ == "__main__":
    print("start!")
    print(FILE_PATH_FOR_DESKTOP)
    print("end!")



__name__ = 'kw618'
__description__ = 'integrated several packages for myself to use'
__version__ = '0.0.1'
__author__ = "Kerwin Lui"
__author_email__ = 'kerwin19950830@gmail.com'
__copyright__ = 'Copyright 2020 Kerwin Lui'



"""
    因为kw618的init中只能导入全局变量/函数/类, 而无法导入类中的函数.
    所以, 其实把该模块作为一个"大的类", 里面都是类中实现某些功能的函数
    所以, docs_2_df 函数, 其实没必要归纳到类中. 这样显得层级很复杂, 而且也不方便外部脚本调用该函数.
"""

import pandas as pd
import numpy as np
import math
import collections
import swifter
import pymongo
import json

import warnings
warnings.filterwarnings("ignore")


def import_data(
    in_file_name="in", end_index=None, field=None, is_df=True,
    in_file_path=None, encoding="gb18030", index_col=None,
    ):
    """
    in:csv文件
    out:df类型/类mongo类型
    function:  csv → df/mongo (默认转出:类mongo)

    notes: in_file_path 的优先级比 in_file_name 高。
    """
    if in_file_path:
        df = pd.read_csv(in_file_path, encoding=encoding, engine='python', index_col=index_col)
    else:
        df = pd.read_csv(FILE_PATH_FOR_DESKTOP+"/{0}.csv".format(in_file_name), encoding=encoding, engine='python', index_col=index_col)
    if is_df:
        return df
    # 1.需要返回的是某个字段的lst格式
    if field:
        field_lst = df[field].values[:end_index] # 得到的是np.array格式
        return list(field_lst) # 用list数据格式来返回
    # 2.返回的是mongo支持的docs
    df = df[:end_index]
    docs = df.T.to_dict().values()
    return docs

def df_2_mongo(df):
    return df.T.to_dict().values() # 即：docs

    #  也可以用于 "mongo → df"
def output_data(
    in_obj, out_file_name="out", ordered_field_lst=None,
    out_file_path=None, output=True, index=False, encoding="gb18030", export_excel=False,
    ):
    """
    in:类mongo/df
    out:csv文件
    function:  1.mongo/df  → csv
               2.mongo → df (这样output设为False即可)

    in_obj:    不管是mongo还是df,自动先转化成df,再用它来转csv

    tips: 如果需要 "mongo → df": output设置为False即可!
    notes: out_file_path 的优先级比 out_file_name 高。

    """

    # 1. 如果不是df类型,先将类mongo的数据,转化成df
    if isinstance(in_obj, pymongo.cursor.Cursor):
        # total_items = []
        # for doc in in_obj:
        #     # items = {i:str(j).strip() for i, j in zip(list(doc.keys()), list(doc.values()))}
        #     # 以下会按照mongo中存着的顺序进行输出!
        #     items = collections.OrderedDict({i:str(j).strip() for i, j in zip(list(doc.keys()), list(doc.values()))})
        #     total_items.append(items)
        # df = pd.DataFrame(total_items)
        df = pd.DataFrame(list(in_obj))
    elif isinstance(in_obj, pd.core.frame.DataFrame):
        df = in_obj

    # 2.确定字段的呈现顺序
    if ordered_field_lst:
        # 如果指定的df字段在df中并不存在,则把该字段remove掉.确保不报错
        for field in ordered_field_lst.copy():
            if field not in df.columns:
                print("字段 {} 不在df中,将其抛弃!".format(field))
                ordered_field_lst.remove(field)
        df = df[ordered_field_lst]  # 指定顺序

    # 3.看是否需要导出csv文件,如果不需要,直接返回df
    if not output:
        return df

    # 4. 最后,将df数据转成csv文件输出
    try:
        if out_file_path:
            if not export_excel:
                df.to_csv(out_file_path, index=index, encoding=encoding)
            else:
                df.to_excel(out_file_path, index=index, encoding=encoding)
        else:
            if not export_excel:
                df.to_csv(FILE_PATH_FOR_DESKTOP+"/{0}.csv".format(out_file_name), index=index, encoding=encoding)
            else:
                df.to_excel(FILE_PATH_FOR_DESKTOP+"/{0}.xlsx".format(out_file_name), index=index, encoding=encoding)
    except Exception as e:
        print(e)
        out_file_name = input("输出文件名出错,请重新键入文件名: ")
        df.to_csv(FILE_PATH_FOR_DESKTOP+"/{0}.csv".format(out_file_name), index=index, encoding=encoding)

    return df


# class KwPd():
#     def __init__(self):
#         pass
#
#     def docs_2_df(self, docs, ordered_field_lst=None):
#         """
#         把mongo的数据转化成df
#         """
#         df = output_data(docs, output=False, ordered_field_lst=ordered_field_lst)
#         return df



def docs_2_df(self, docs, ordered_field_lst=None):
    """
    把mongo的数据转化成df
    """
    df = output_data(docs, output=False, ordered_field_lst=ordered_field_lst)
    return df


# stackoverflow 白嫖来的函数，hhh
def read_mongo(collection, query={}, no_id=True):
    """ Read from Mongo and Store into DataFrame """


    # Make a query to the specific DB and Collection
    cursor = db[collection].find(query, {"_id":0})

    # Expand the cursor and construct the DataFrame
    df =  pd.DataFrame(list(cursor))

    # Delete the _id
    # if no_id:
    #     del df['_id']

    return df






def avg(lst):
    if isinstance(lst, list):
        if len(lst) <1:
            # raise myError("元素小于1!")
            return 0
    elif isinstance(lst, type(pd.Series())):
        if lst.size <1:
            # raise myError("元素小于1!")
            return 0
    sum = 0
    for count, e in enumerate(lst):
        # print(count, e)
        sum += int(float(e))
    lst_avg = sum/(count+1)
    # print(lst_avg)
    return int(lst_avg)


def merge_df(
    x_name, y_name, out_file_name="out",
    is_df=None, join_field="house_id", output=True):
    """
    function: 不仅可以合并df/csv, 还附带输出csv的功能
    """
    print(">>>1")
    if not is_df:
        # 如果 不是df， 就把这个当做文件名，导入
        x_df = import_data(x_name, is_df=True)
        y_df = import_data(y_name, is_df=True)
    else:
        # 如果 是df， 就直接把传入的x、y当做 df对象来使用
        x_df = x_name
        y_df = y_name
    print(">>>2")
    # pd.merge() 返回的不是df类型，而是function类型。 但这个function可以使用to_csv导出文件
    #  ??????   什么情况？ 之前测试的时候返回的不是df对象，现在测试发现又确实是df对象了。。。见鬼！
    merged_df = pd.merge(x_df, y_df, how="left", on=join_field)
    if not output:
        return merged_df
    print(">>>3")
    merged_df.to_csv(FILE_PATH_FOR_DESKTOP+"/{0}.csv".format(out_file_name), index=False, encoding="gb18030")
    print("合并成功!")

# merge_df("aaa", "bbb", out_file_name="zzzz")
# exit()


# def k_top(lst, top=1):
#     if isinstance(lst, list):
#         if len(lst) <1:
#             # raise myError("元素小于1!")
#             return 0
#     elif isinstance(lst, type(pd.Series())):
#         if lst.size <1:
#             # raise myError("元素小于1!")
#             return 0
#
#     lst = sorted(lst)
#     return lst[top-1]


class NumpyEncoder(json.JSONEncoder):
    """ Special json encoder for np types """
    def default(self, obj):
        if isinstance(obj, (np.int_, np.intc, np.intp, np.int8,
                            np.int16, np.int32, np.int64, np.uint8,
                            np.uint16, np.uint32, np.uint64)):
            return int(obj)
        elif isinstance(obj, (np.float_, np.float16, np.float32,
                              np.float64)):
            return float(obj)
        elif isinstance(obj, (np.ndarray,)):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)










def k_divide(lst, piece=5):
    """
    function: 按lst从小到大的顺序, 等分成piece份 小lst 返回
    """
    if isinstance(lst, list):
        if len(lst) <1:
            # raise myError("元素小于1!")
            return 0
    elif isinstance(lst, type(pd.Series())):
        if lst.size <1:
            # raise myError("元素小于1!")
            return 0

    lst = sorted(lst)
    # 1. 打印原lst
    print(lst)
    node_order_lst = []
    node_lst = []
    for count in range(1, piece):
        node_order_value = round(len(lst) * (1/piece) * count) - 1 # 减一别忘了 (另外,这里返回的是顺序值,不是真实值)
        node_order_lst.append(node_order_value)
        node_lst.append(lst[node_order_value])
    # 2. 打印分好piece后的, 节点的顺序
    print(node_order_lst) # 是顺序
    print("值的lst: {}".format(node_lst)) # 是值

    piece_dict = {}
    count = 0
    while True:
        if count == piece:
            break
        elif count == 0:
            piece_dict.update({count+1 : lst[ : node_order_lst[count]+1]})
        elif count == piece-1:
            piece_dict.update({count+1 : lst[node_order_lst[count-1]+1 : ]})
        else:
            piece_dict.update({count+1 : lst[node_order_lst[count-1]+1 : node_order_lst[count]+1]})
        count += 1
    # 3. 打印根据上面的顺序, piece等拆分了lst后的dict
    print(piece_dict)
    return node_lst
    # return piece_dict


    # piece_lst = [] count = 0
    # while True:
    #     if count == piece:
    #         break
    #     elif count == 0:
    #         piece_lst.append(lst[ : node_order_lst[count]+1])
    #     elif count == piece-1:
    #         piece_lst.append(lst[node_order_lst[count-1]+1 : ])
    #     else:
    #         piece_lst.append(lst[node_order_lst[count-1]+1 : node_order_lst[count]+1])
    #     count += 1
    # # 3. 打印根据上面的顺序, piece等拆分了lst后的lst
    # print(piece_lst)
    # return piece_lst

# k_divide([3, 4, 5, 7, 2, 4, 46, 6, 7, 84, 4,5], 5)





if __name__ == '__main__':
    print("start!")
    df = import_data("业务反馈调价", is_df=True)
    print(df)
    print(df.shape)
    print("end!")



class AbuyunProxies():

    # 代理服务器
    proxyHost = "http-dyn.abuyun.com"
    proxyPort = "9020"

    # 代理隧道验证信息
    proxyUser = "H9E6383C4ONW1AOD"
    proxyPass = "11844987BD73DCAD"

    proxyMeta = "http://%(user)s:%(pass)s@%(host)s:%(port)s" % {
      "host" : proxyHost,
      "port" : proxyPort,
      "user" : proxyUser,
      "pass" : proxyPass,
    }

    proxies = {
        "http"  : proxyMeta,
        "https" : proxyMeta,
    }




"""
notes:
    该文件作为所有ziru相关的、可重复使用的api接口！！
    初衷：写了太多自如相关的函数，但是都不够标准化，没有统一的api规范，
         导致重复造了多个轮子来实现类似的功能，低效不可持续利用！！
         现在，想要通过“规范化、标准化”，让类似的功能集成在一个万能的函数中！！
         （可以把性能作为代价，但一定要让函数变得容易调用、方便记忆）

exec_cmd:
    ## 快速编辑标准价对应表
    vim /root/kerwin/MyBox/MyKernel/python_default_path/ziru/estate_std_price.csv
    ps aux
    kill -s 9 <pid>
    nohup python3 -u /root/kerwin/MyBox/MyCode/Business/Ziru/97_Exec_Script/主动调价_0730/Approve_Pricing.py > /root/Approve_Pricing.log &
"""



# TODO:  构建 ”标准户型“ 字段，通过pandas矢量化的方法来实现，性能可以提高上百倍！！！
            # 所以不到万不得已，尽量不要使用apply来逐行执行！！


# TODO:  求标准价的函数 --->> 升级版（可以进而计算大盘系数、6.0系数、成本系数、特殊系数？？等除了空置系数外的系数！！）



# 价格名称由大到小：楼盘标准价 >  户型标准价 >  id标准价  >  id空置标准价 > 修正尾数价格
# estate_std_price > std_price > id_std_price > id_idle_std_price > modified_price

def normalized_filed_for_huxing(df):
    """
    required_fields: 楼盘名称、面积、卧室名称、是否有独卫、是否有阳台
    """
    # 标准化"友家房源"的id户型字段
    df["面积"] = df["面积"].astype("int").astype("str")
    df["朝向"] = df["卧室名称"].str.extract(r"(\w+)卧")
    df["朝向"] = df["朝向"].str.replace("^西南$", "南", regex=True)
    df["朝向"] = df["朝向"].str.replace("^东南$", "南", regex=True)
    df["朝向"] = df["朝向"].str.replace("^西北$", "东西", regex=True)
    df["朝向"] = df["朝向"].str.replace("^东北$", "东西", regex=True)
    df["朝向"] = df["朝向"].str.replace("^南北$", "东西", regex=True)
    df["朝向"] = df["朝向"].str.replace("^东$", "东西", regex=True)
    df["朝向"] = df["朝向"].str.replace("^西$", "东西", regex=True)
    df["id户型"] = df["楼盘名称"] +"-"+ df["是否有独卫"] +"-"+ df["是否有阳台"] +"-"+ df["朝向"] +"-"+ df["面积"]

    return df


def revise_tail_number(m_price, revise_type="medium"):
    """
    function: 把价格的尾数按照规则变成 3、6、9结尾的价格
    revise_type:
        round_up/round_down/medium
    """

    if pd.api.types.is_number(m_price):
        m_price = int(m_price)
        remainder = m_price % 100
        if revise_type == "medium":
            if remainder < 10:
                modified_price = int(m_price/100)*100 - 10
            elif remainder < 45:
                modified_price = int(m_price/100)*100 + 30
            elif remainder < 75:
                modified_price = int(m_price/100)*100 + 60
            elif remainder < 100:
                modified_price = int(m_price/100)*100 + 90
        elif revise_type == "round_up":
            if remainder <= 30:
                modified_price = int(m_price/100)*100 + 30
            elif remainder <= 60:
                modified_price = int(m_price/100)*100 + 60
            elif remainder <= 90:
                modified_price = int(m_price/100)*100 + 90
            elif remainder < 100:
                modified_price = int(m_price/100)*100 + 130
        elif revise_type == "round_down":
            if remainder < 30:
                modified_price = int(m_price/100)*100 - 10
            elif remainder < 60:
                modified_price = int(m_price/100)*100 + 30
            elif remainder < 90:
                modified_price = int(m_price/100)*100 + 60
            elif remainder < 100:
                modified_price = int(m_price/100)*100 + 90
    else:
        print(m_price)
        modified_price = "无法修正其尾数价格"

    return modified_price

# revise_tail_number(3.33)
# sys.exit()




# TODO:  每天早上先把价格梳理一遍，之后的续约、标准价、二次调价都可以参考这部分总的价格参考表！！！
# TODO:  后续在业务发起调价的时候，也可以在邮件中展示这部分信息（最全面的价格参考表）（甚至可以包括dk价格）
# TODO:  也就是完成当年的设想：给我一个house_id，实现一秒返回所有相关的价格字段！！
            ## 包括但不局限于： 目前调价表中的所有字段、蛋壳相关的价格、待租量、补贴量？、url跳转到类似户型的价格
            ##（面积上下2平内？或者直接呈现该楼盘的所有户型？）











        #####















#



import re
import json
import collections
import time
import pymongo
import redis



# redis的 pipe管道事务操作:
# 1. pipe为事务管道,必须要execute后才会在服务器端真正执行
# 2.当管道中使用watch开启检测后,到multi语句(或者execute语句)前,会让pipe处于"非事务阶段",
#   可以获取、删除redis服务器中的值.  (意味着此阶段的pipe.set()不需要execute())
# 3. 当pipe.execute()执行时,会把储存在pipe管道中"滞后执行"的事务操作全部一次性完成.
# 4.在"管道事务"阶段,打印出来的都是对象(在后面execute后接受str结果); 而在"非事务"阶段,返回的是字符.

# def lst_to_zset(lst=[4, 1, 88], key_name="queue"):
#     m_ = zip(lst, range(len(lst)))
#     all_z_dict = {i:j for i, j in m_}
#     r.delete(key_name)
#     r.zadd(key_name, all_z_dict)



# 应对业务需求，快速匹配出他们想要的消化数据
def merge_mongo(
    left_table_name="ziru_stock", right_table_name="ziru_name_list", left_join_field="所属业务组",
    right_join_field="ziru_zone", conditions_dict={}, project_dict={"_id":0}, db=None,
    ):
    """
    notes:
        1. project_dict的功能是表示哪些字段需要显示，哪些不要显示。但是只有“_id”可以为0，其他只能标记为1.
        2. 右连接的字段前必须加上matched_field才行
        3. 返回得到的 all_join_docs 中， 右连接得到的所有内容都包含在 matched_field 字段中，以dict形式存在
    """

    if db is None:
        client = pymongo.MongoClient("127.0.0.1")
        db = client["zufang_001"]

    # 输入接口：
    # left_table_name
    # right_table_name

    # 管道筛选：
    pipeline = [
                {
                    "$lookup":
                    {
                        "from":right_table_name,
                        "localField":left_join_field,
                        "foreignField":right_join_field,
                        "as":"matched_field",
                    }
                },
                {
                    "$match": conditions_dict,
                },
                {
                    "$project": project_dict,
                }
                ]

    all_join_docs = db[left_table_name].aggregate(pipeline)
    return all_join_docs



# with r.pipeline() as pipe:
#     while True:
#         try:
#             # 监视我们需要修改的键
#             queue_ = "lll"
#             # pipe.rpush(queue_, *[3, 4, 5])
#             # 进入"非事务"阶段
#             pipe.watch(queue_)
#             pipe.rpush(queue_, *[44, 88])
#
#             # 进入"事务"阶段
#             pipe.multi()
#             x = pipe.lrange(queue_, 0, -1)
#             print(x)
#             # print(pipe.rpop(queue_))
#             ss = pipe.execute()
#             print(ss)
#             break
#
#         except redis.WatchError:
#             # 如果其他客户端在我们 WATCH 和 事务执行期间，则重试
#             # pipe.unwatch()
#             pipe.reset() # unwatch和ureset的作用应该差不多
#             print("error")
#             break
#


def main():
    pass
    # r.sadd("bnm", print(3))
    # r.lpush("aabb", *[1, 2, 3])
    # print(r.lrange("a", 0, -1))
    # print(r.smembers("crawl_cost_price:crawled_queues"))



if __name__ == '__main__':
    print("start test!")
    main()
    print("\n\n\neverything is ok!")



"""
    导入顺序:
        1. 第三方库 2.本地类 3.本地函数 4.本地常量 5.本地变量
"""

# 库包版本相关
from kw618._version import (
    __name__, __description__, __version__, __author__,
)

# 导入常用的固定路径(多平台通用)
from kw618._file_path import *



# kw618库包中需要用到的所有第三方库
hard_dependencies = (
    # requests相关
    "sys", "os", "retry", "traceback", "pysnooper", "user_agent", "random", "requests",
    "threading", "multiprocessing", "scrapy", "urllib", "smtplib", "uuid", "email", "execjs",
    "copy", "exchangelib", "urllib3", "selenium",
    # pandas相关
    "numpy", "pandas", "math", "collections", "swifter", "pymongo", "warnings",
    # pymongo相关
    "re", "json", "time", "pymongo", "redis",
    )

# 判断依赖包是否存在
missing_dependencies = []
for dependency in hard_dependencies:
    try:
        if dependency == "numpy":  # 1. 使用语句的方式import
            import numpy as np
        elif dependency == "pandas":
            import pandas as pd
        elif dependency == "warnings":
            import warnings
            warnings.filterwarnings("ignore")
        else:
            # __import__(dependency)  # 2. 使用函数方式import: 可以用str类型作为参数传入
            exec("import {}".format(dependency))
    except ImportError as e:
        missing_dependencies.append("{0}: {1}".format(dependency, str(e)))
if missing_dependencies:
    raise ImportError(
        "无法导入所需依赖包:\n" + "\n".join(missing_dependencies)
    )
del hard_dependencies, dependency, missing_dependencies



# 导入所有模块中的所有全局变量/函数/类
# (这些都是"kw618"这个 package namespace 底下的 API )
    ## 1. 第三方库封装模块
        ##### (pandas的__init__.py 中 即使要全部导入也是一个个导,不会使用*)
from kw618.k_pandas.utils_pandas import *
from kw618.k_pymongo.utils_pymongo import *
from kw618.k_pymysql.utils_pymysql import *
from kw618.k_requests.utils_requests import *
from kw618.k_requests.my_cookies import *
    ## 2. 工作相关模块
from kw618.ziru.std_price_system import *
from kw618.ziru.utils_ziru import *
    ## 3. 导入自己写的外部脚本 (不存在于kw618库内的) (需要先添加python默认搜索路径)
    ## 把各种需要的、重要的包(package) 加入到python的默认搜索路径中 (导入个人使用的package)
if os.path.exists(f"{FILE_PATH_FOR_ZIRU_CODE}/97_Exec_Script"):
    sys.path.append(f"{FILE_PATH_FOR_ZIRU_CODE}/97_Exec_Script")
    from 获取house_id的各类参考价格_0730.Get_Stdprice import *
    from 爬取自如price系统最低成本价_0724.Crawl_Costprice import crawl_all_costprice
if os.path.exists(f"{FILE_PATH_FOR_ZIRU_CODE}/98_Utlis"):
    sys.path.append(f"{FILE_PATH_FOR_ZIRU_CODE}/98_Utlis")
    # from k01_Split_Table import *  # 不太常用，暂不开启



# 导入通用变量/常量
    ## pandas
today_obj = pd.to_datetime("today")
today_date = today_obj.strftime("%Y-%m-%d") # 转成“2019-02-28”这样的str形式
this_time = today_obj.strftime("%X") # 转成“2019-02-28”这样的str形式
yesterday_obj = today_obj - pd.to_timedelta("1 d")
yesterday_date = yesterday_obj.strftime("%Y-%m-%d") # 转成“20190228”这样的str形式
df = import_data(in_file_path=f"{FILE_PATH_FOR_KW618}/k_pandas/df_test.csv")

    ## pymongo
client = pymongo.MongoClient(HOST)
db = client["zufang_001"]
remote_client = pymongo.MongoClient("121.40.240.153")
remote_db = remote_client["zufang_001"]

    ## redis
r = redis.StrictRedis(host="localhost", port=6379, db=0, decode_responses=True)

    ## requests
req = myRequest().req




































##



from k_functions import *

"""
input: [楼盘坐标.csv]
output: [附近1公里楼盘.csv]
function:
    自己搭建的一个异步爬虫框架，很完美的应用！！（可作为模板，之后重复利用）
api:
    [in_file_name,
    out_file_name
    DISTANCE (自定义周边距离)]

tips: []
warning: []
"""
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

db = client["zufang_001"]
req = myRequest().req


# @decorate_error_log
def req_and_parse_detail_page(url):
    # 打印正在运行的函数名
    print(sys._getframe().f_code.co_name)
    # 请求url，获得response对象
    response = req(url, selector=True)
    try:
        item = {}
        house_id = re.findall(r".*?/(\w+).html", url)[0]
        # 针对每个房源的个性标签
        room_title = response.xpath("/html/body/div[3]/div[1]/div[2]/div[2]/div[2]//text()").extract()
        room_title = "".join(room_title)

        room_list_box = response.xpath("/html/body/div[3]/div[1]/div[2]/div[2]/div[4]")
        print(room_list_box)
        for e in room_list_box:
            housing_area = e.xpath("./div[1]/div[1]/label/text()").re(r"约(\d+)")[0]
            house_numbering = e.xpath("./div[1]/div[2]/label/text()").re(r"编号：(\d+.*?\w)")[0]
            shi = e.xpath("./div[1]/div[3]/label/text()").re(r"户型：(\d+)室(\d+)卫")[0]
            wei = e.xpath("./div[1]/div[3]/label/text()").re(r"户型：(\d+)室(\d+)卫")[1]


        # 把所有字段映射放入item字典中
        item["house_id"] = house_id

        item["housing_area"] = housing_area
        item["house_numbering"] = house_numbering
        item["shi"] = shi
        item["wei"] = wei

        print(item, "\n\n")
        return item

    except Exception as e:
        tb_txt = traceback.format_exc(limit=1)
        print("detail_url 页面解析出错啦：\n", tb_txt)
        with open("detail_url_error.txt", "a", encoding="utf8") as f:
            f.write("{0}\ndetail_url 页面解析出错啦:\n{1}\n{2}\n\n\n".format(time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()), url, tb_txt))

# req_and_parse_detail_page(url="https://www.danke.com/room/2086319156.html")




def get_data_to_produce():
    docs = db["danke_stock"].find({"estate":None})
    url_lst = []
    for doc in docs:
        url = doc.get("detail_url")
        # print(url)
        url_lst.append(url)
    print(url_lst)
    print(len(url_lst))
    return url_lst


def consume(e):
    # 这里的e 使用的是url
    url = e
    # 获取danke详情页中的所有字段信息，得到item
    item = req_and_parse_detail_page(url)
    # 更新danke_stock表
    house_id = item["house_id"]
    db["danke_stock"].update_one({"house_id":house_id}, {"$set":item})
    # 手动延迟0.2秒， 防止被反爬
    time.sleep(0.8)


def main():
    """
    in: get_data_to_produce函数；  queue_name;   consume函数(核心是还是 req_and_parse_detail_page函数--用来‘请求’和’解析’)
    function: 获取所有待爬元素e，使用consume进行消费

    【自己搭建的异步爬虫的整体框架】

    """
    # 1. 获取所有待爬元素 (包含所有的 'e' )
    url_lst = get_data_to_produce()
    # 2. 定义队列名
    queue_name = "get_detail_page"
    # 3. 生产待爬队列：生产者将所有元素放入redis的待爬队列中。
    k_produce(lst=url_lst, queue_name=queue_name)
    # 4. 消费待爬队列：建立多线程池，循环消费redis待爬队列中的所有元素 (consume_func函数循环获取所有的‘e’)
    ### 其中，已经建立了防漏爬的安全机制！
    ### consume_func 才是真正执行任务的消费函数
    k_consume_pool(consume_func=consume, queue_name=queue_name, thread_num=5, time_sleep=1)
    ## tips: 上面生产者传入reids的e， 与后面消费者消费的e， 必须要保证"e"一致！
    ##        (可以都是id，也可以都是url)  (本脚本使用url)



if __name__ == '__main__':
    print("start test!")
    main()
    print("\n\n\neverything is ok!")
